#include <TIPbusClient.h>

///////////////////////////////////////////////////////////////////////////////
/**Class constructor.
*/
TIPbusClient::TIPbusClient(QObject* parent): QObject(parent)
{
timeoutVal=10;
timeoutTimer=new QTimer(0);
timeoutTimer->setSingleShot(true);
timeoutTimer->setInterval(timeoutVal);

reqThread=new TreqThread;
thread = new QThread();

sendTimer=new QTimer(0);
sendTimer->setInterval(1);

connect(sendTimer,SIGNAL(timeout()),reqThread,SLOT(processRequest()));
//connect(thread,SIGNAL(started()),sendTimer,SLOT(start()));
connect(this,SIGNAL(startTimer()),sendTimer,SLOT(start()));
connect(this,SIGNAL(startTimer()),reqThread,SLOT(processRequest()));
connect(this,SIGNAL(stopTimer()),sendTimer,SLOT(stop()));

connect(timeoutTimer,SIGNAL(timeout()),reqThread,SLOT(manageTimeoutError()));
//connect(this,SIGNAL(startTimeout(int)),timeoutTimer,SLOT(start(int)));
connect(this,SIGNAL(startTimeout()),timeoutTimer,SLOT(start()));
connect(this,SIGNAL(stopTimer()),timeoutTimer,SLOT(stop()));

//can't execute in this slot, must restart timer with previous timeout
connect(reqThread,SIGNAL(rearmTimeoutTimer()),timeoutTimer,SLOT(start()));
connect(reqThread,SIGNAL(stopTimeoutTimer()),timeoutTimer,SLOT(stop()));

qRegisterMetaType<TreqParam >("TreqParam");
connect(this,SIGNAL(addControlRequest(TreqParam)),reqThread,SLOT(addControlRequest(TreqParam)));

connect(this,SIGNAL(connectClientSig(QString, int)),reqThread,SLOT(connectClient(QString, int)));

reqThread->moveToThread(thread);
reqThread->TC_socket->moveToThread(thread);

sendTimer->moveToThread(thread);
timeoutTimer->moveToThread(thread);

thread->start();
}
///////////////////////////////////////////////////////////////////////////////
/**Class destructor.
*/
TIPbusClient::~TIPbusClient()
{
}
///////////////////////////////////////////////////////////////////////////////
void TIPbusClient::connectClient(QString addStr_in, int port_in)
{
reqThread->dispatchMutex.lock(); //check no command in processing
reqThread->dispatchMutex.unlock();

reqThread->connectMutex.lock();
emit connectClientSig(addStr_in,port_in);

//return to main loop when connected
//cout << "unlock" << endl;
//try lock to get hand on process
reqThread->connectMutex.lock();
reqThread->connectMutex.unlock();
}
///////////////////////////////////////////////////////////////////////////////
void TIPbusClient::setTimeout(int val)
{
timeoutVal=val;
timeoutTimer->setInterval(timeoutVal);
}
///////////////////////////////////////////////////////////////////////////////
void TIPbusClient::read_std(int add, quint32 *buff,int size)
{
TreqParam reqParam;
reqParam.type=_read_std;
reqParam.baseadd=add;
reqParam.ptr=buff;
reqParam.param1=size;
reqParam.param2=0;

emit addControlRequest(reqParam);
}
///////////////////////////////////////////////////////////////////////////////
void TIPbusClient::read_ni(int add, quint32 *buff,int size)
{
TreqParam reqParam;
reqParam.type=_read_ni;
reqParam.baseadd=add;
reqParam.ptr=buff;
reqParam.param1=size;
reqParam.param2=0;

emit addControlRequest(reqParam);
}
///////////////////////////////////////////////////////////////////////////////
void TIPbusClient::write_std(int add, quint32 *buff,int size)
{
TreqParam reqParam;
reqParam.type=_write_std;
reqParam.baseadd=add;
reqParam.ptr=buff;
reqParam.param1=size;
reqParam.param2=0;

emit addControlRequest(reqParam);
}
///////////////////////////////////////////////////////////////////////////////
void TIPbusClient::write_ni(int add, quint32 *buff,int size)
{
TreqParam reqParam;
reqParam.type=_write_ni;
reqParam.baseadd=add;
reqParam.ptr=buff;
reqParam.param1=size;
reqParam.param2=0;

emit addControlRequest(reqParam);
}
///////////////////////////////////////////////////////////////////////////////
/** Read modify write bit.
Algorithm used is X<=(X & ANDterm) | ORterm. Value returned is the one before modificiation.
*/
void TIPbusClient::rmw_bit(int add, quint32 *buff,quint32 ANDterm,quint32 ORterm)
{
TreqParam reqParam;
reqParam.type=_rmw_bit;
reqParam.baseadd=add;
reqParam.ptr=buff;
reqParam.param1=ANDterm;
reqParam.param2=ORterm;

emit addControlRequest(reqParam);
}
///////////////////////////////////////////////////////////////////////////////
/** Read modify sum.
Algorithm used is X<=(X + A). Value returned is the one before modificiation.
*/
void TIPbusClient::rmw_sum(int add, quint32 *buff,quint32 A)
{
TreqParam reqParam;
reqParam.type=_rmw_sum;
reqParam.baseadd=add;
reqParam.ptr=buff;
reqParam.param1=A;
reqParam.param2=0;

emit addControlRequest(reqParam);
}
///////////////////////////////////////////////////////////////////////////////
void TIPbusClient::dispatch()
{
//emit startTimeout();

//cout << "lock()" << endl;
reqThread->dispatchMutex.lock();

TreqParam reqParam;
reqParam.type=_dispatch;
reqParam.baseadd=0;
reqParam.ptr=0;
reqParam.param1=0;
reqParam.param2=0;

emit addControlRequest(reqParam);
emit startTimer();

//try lock to wait process
reqThread->dispatchMutex.lock();
reqThread->dispatchMutex.unlock();
//cout << "unlock()" << endl;
//cout << endl;
emit stopTimer();
}
