#ifndef TIPBUS_TRANSACTION_H
#define TIPBUS_TRANSACTION_H
#include <QtGlobal> 
#include <QMetaType>

#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>
#include <limits>
#include <string>

using namespace std;

enum TreqType {_read_std=0,_read_ni=2,_write_std=1,_write_ni=3,_rmw_bit=4,_rmw_sum=5,_done,_dispatch};

class TreqParam
{
public:
	int type;
	int baseadd;
	quint32 *ptr;
	int param1;
	int param2;
};

Q_DECLARE_METATYPE(TreqType);

#define _IPBusType(w) (w&0xF)/**<packet type.*/
#define _IPBusBOQ(w) ((w>>4)&0xF)/**<Byte Order qualifier.*/
#define _IPBusPacketID(w) ((w>>8)&0xFFFF)/**<Packet ID.*/
#define _IPBusProtVer(w) ((w>>28)&0xF)/**<Protocol version.*/

#define _TransInfoCode(w) (w&0xF)/**<information code.*/
#define _TransTypeID(w) ((w>>4)&0xF)/**<transaction type.*/
#define _TransWords(w) ((w>>8)&0xFF)/**<Transfert size (max 255).*/
#define _TransID(w) ((w>>16)&0xFFF)/**<Transaction ID.*/
#define _TransProtVer(w) ((w>>28)&0xF)/**<Protocol version.*/

class TIPbusTransaction {
public:
	TIPbusTransaction(int int_size);
	~TIPbusTransaction();
	int getAvailableRxLength();
	int getAvailableTxLength();
	uint getFrameSize();
	quint32 *dataFrame_ptr();
	quint32 *transactionPtr_ptr(int trans);
	bool appendToTransaction(int packID,TreqParam &param); 
	void clean();
	quint32 expectedPacketID();
	uint getTransactionCount();
	uint getExpectedRxLength();

private:
	int maxPayloadLength;
	int expectedRxLength;
	uint transactionCount;
	int currentTxLength;
	quint32 packetID;
	bool IDSet;
	bool setPacketID(quint32 id);
	void readTransaction(int type,int add, quint32 *buff,int size);
	void writeTransaction(int type,int add, quint32 *buff,int size);
	quint32 *WORD;
	quint32 **buf_ptr;
};

#endif

