#ifndef _REQTHREAD_H_
#define _REQTHREAD_H_

#include <QThread>
#include <QTimer>
#include <QMutex>
#include <QQueue>
#include <QUdpSocket>
#include <QHostAddress>
#include <QWaitCondition>
#include <QDateTime>

#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>
#include <limits>
#include <string>

#include <TIPbusTransaction.h>
#include <packets.h>
using namespace std;

/**Exception class.
 * Allows to pass error message to the upper level.
 * */

class TIPbusError
{
public:
	TIPbusError(QString Mess);
	bool fatalError;
	uint errorProcessed;
	QString Message;
};

/** Thread managing the acquisition and the histogram building and the root tree filling.
This is the main thread managing the acquisition task. 
*/
class TreqThread : public QObject
{
 	Q_OBJECT

private:
	QString addStr;
	QHostAddress hostAdd;
	TstatusPacket statusPacket;
	TIPbusTransaction **IPbusTransactions;
	bool firstStatusPacket;
	quint32 datagram[9000];
	int receivedWords;
	quint32 status_header;
	bool inResponseRecovery;
	bool inRequestRecovery;
	quint32 reissueCount;
	qint32 timeoutStatusGearBox;

   	quint16 port;
	QQueue <int> availTxTrans;
	QQueue <TreqParam> reqQueue;
	QQueue<int> recvTrans;

	int packetID;
	int getCurTransBuff();
	uint nResponseBuffers;
	uint errorProcessing;
	bool timeoutStatusRequest;

	void requestStatusPacket();
	void sendRequest(int transToXmit,bool isReissue=false);
	void processInitialStatusPacket();
	void parseResponse(int slot);
	uint networkMTU;
	TreqParam curParam;
	bool requestInProcess;
	void releaseTxSlot(int slot);
	void errorRecovery();
	void resendPacket(quint16 packID);

public:
	TreqThread(QObject *parent = 0);
	~TreqThread();
	QUdpSocket *TC_socket;
	QMutex dispatchMutex;
	QMutex connectMutex;

public slots:
	void connectClient(QString addStr_in, int port_in);
	void addControlRequest(TreqParam reqParam);
	void processRequest();
	void manageTimeoutError();

private slots:
	void ByeMessage();
	void getPacket();
	void ConnectMessage();
	void printState(QAbstractSocket::SocketState);

signals:
	void  Message(QString);
	void rearmTimeoutTimer();
	void stopTimeoutTimer();

 };
#endif
