-------------------------------------------------------------------------------
-- FRAME 0 : requete ARP
-- FRAME 1 : requete ping
-- FRAME 2 : requete IPbis status
-- FRAME 3 : requete read (packetid=0), 4 read depuis adresse 0
-------------------------------------------------------------------------------

library std;
use std.textio.all;
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_textio.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity tb_top_ipbus_lpsc_basex is
end tb_top_ipbus_lpsc_basex;

architecture simu of tb_top_ipbus_lpsc_basex is 
component top_ipbus_lpsc_basex
generic (
	IN_SIMULATION     : boolean :=false;
	SIM_GTPRESET_SPEEDUP : natural :=0;
	SIMUL_DST_MAC_NUMBER  : std_logic_vector(47 downto 0):=x"AAAA_BBBB_CCCC";
	CLOCK_FREQ			: integer := 125000000;							-- freq of data_in_clk -- needed to timout cntr
	ARP_TIMEOUT			: integer := 60;									-- ARP response timeout (s)
	ARP_MAX_PKT_TMO	: integer := 5;									-- # wrong nwk pkts received before set error
	MAX_ARP_ENTRIES 	: integer := 255									-- max entries in the ARP store
	);
port(
		sysclk_p, sysclk_n: in STD_LOGIC;
		gtp_clkp, gtp_clkn: in std_logic;
		gtp_txp, gtp_txn: out std_logic;
		gtp_rxp, gtp_rxn: in std_logic;
		sfp_los: in std_logic;
		user_sma_gpio_n : out std_logic;
		user_sma_gpio_p : out std_logic;
		leds: out STD_LOGIC_VECTOR(3 downto 0);
		dip_switch: in std_logic_vector(3 downto 0)
	);
end component;
------------------------------------------------------------------------------
-- types to support frame data
------------------------------------------------------------------------------
-- Tx Data and Data_valid record
type data_typ is record
  data : bit_vector(7 downto 0);        -- data
  valid : bit;                          -- data_valid
  xmit : bit;                          -- data_valid
  error : bit;                          -- data_error
end record;
type frame_of_data_typ is array (natural range <>) of data_typ;

-- Tx Data, Data_valid and underrun record
type tri_mode_eth_mac_v5_3_frame_typ is record
  columns   : frame_of_data_typ(0 to 125);-- data field
  bad_frame : boolean;                   -- does this frame contain an error?
end record;
type frame_typ_ary is array (natural range <>) of tri_mode_eth_mac_v5_3_frame_typ;

constant TB_MODE                  : string := "DEMO";
signal an_adv_config_vector0:std_logic_vector(15 downto 0);

------------------------------------------------------------------------------
-- Stimulus - Frame data
------------------------------------------------------------------------------
-- The following constant holds the stimulus for the testbench. It is
-- an ordered array of frames, with frame 0 the first to be injected
-- into the core transmit interface by the testbench.
------------------------------------------------------------------------------
constant frame_data : frame_typ_ary := (
-------------
-- Frame 0
-------------
0          => (
  columns  => (
0        => ( DATA => X"ff", VALID => '1', xmit => '1', ERROR => '0'),
1        => ( DATA => X"ff", VALID => '1', xmit => '1', ERROR => '0'),
2        => ( DATA => X"ff", VALID => '1', xmit => '1', ERROR => '0'),
3        => ( DATA => X"ff", VALID => '1', xmit => '1', ERROR => '0'),
4        => ( DATA => X"ff", VALID => '1', xmit => '1', ERROR => '0'),
5        => ( DATA => X"ff", VALID => '1', xmit => '1', ERROR => '0'),
6        => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
7        => ( DATA => X"30", VALID => '1', xmit => '1', ERROR => '0'),
8        => ( DATA => X"48", VALID => '1', xmit => '1', ERROR => '0'),
9        => ( DATA => X"7a", VALID => '1', xmit => '1', ERROR => '0'),
10       => ( DATA => X"e9", VALID => '1', xmit => '1', ERROR => '0'),
11       => ( DATA => X"63", VALID => '1', xmit => '1', ERROR => '0'),
12       => ( DATA => X"08", VALID => '1', xmit => '1', ERROR => '0'),
13       => ( DATA => X"06", VALID => '1', xmit => '1', ERROR => '0'),
14       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
15       => ( DATA => X"01", VALID => '1', xmit => '1', ERROR => '0'),
16       => ( DATA => X"08", VALID => '1', xmit => '1', ERROR => '0'),
17       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
18       => ( DATA => X"06", VALID => '1', xmit => '1', ERROR => '0'),
19       => ( DATA => X"04", VALID => '1', xmit => '1', ERROR => '0'),
20       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
21       => ( DATA => X"01", VALID => '1', xmit => '1', ERROR => '0'),
22       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
23       => ( DATA => X"30", VALID => '1', xmit => '1', ERROR => '0'),
24       => ( DATA => X"48", VALID => '1', xmit => '1', ERROR => '0'),
25       => ( DATA => X"7a", VALID => '1', xmit => '1', ERROR => '0'),
26       => ( DATA => X"e9", VALID => '1', xmit => '1', ERROR => '0'),
27       => ( DATA => X"63", VALID => '1', xmit => '1', ERROR => '0'),
28       => ( DATA => X"c0", VALID => '1', xmit => '1', ERROR => '0'),
29       => ( DATA => X"a8", VALID => '1', xmit => '1', ERROR => '0'),
30       => ( DATA => X"64", VALID => '1', xmit => '1', ERROR => '0'),
31       => ( DATA => X"09", VALID => '1', xmit => '1', ERROR => '0'),
32       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
33       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
34       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
35       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
36       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
37       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
38       => ( DATA => X"c0", VALID => '1', xmit => '1', ERROR => '0'),
39       => ( DATA => X"a8", VALID => '1', xmit => '1', ERROR => '0'),
40       => ( DATA => X"64", VALID => '1', xmit => '1', ERROR => '0'),
41       => ( DATA => X"10", VALID => '1', xmit => '1', ERROR => '0'),
42       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
43       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
44       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
45       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
46       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
47       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
48       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
49       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
50       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
51       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
52       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
53       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
54       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
55       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
56       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
57       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
58       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
59       => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
others  => ( DATA => X"00", VALID => '0', xmit => '0', ERROR => '0')),
  -- No error in this frame
  bad_frame => false),


-------------
-- Frame 1
-------------
1          => (
  columns  => (
0	 => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
1	 => ( DATA => X"0d", VALID => '1', xmit => '1', ERROR => '0'),
2	 => ( DATA => X"db", VALID => '1', xmit => '1', ERROR => '0'),
3	 => ( DATA => X"a1", VALID => '1', xmit => '1', ERROR => '0'),
4	 => ( DATA => X"15", VALID => '1', xmit => '1', ERROR => '0'),
5	 => ( DATA => X"30", VALID => '1', xmit => '1', ERROR => '0'),
6	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
7	 => ( DATA => X"30", VALID => '1', xmit => '1', ERROR => '0'),
8	 => ( DATA => X"48", VALID => '1', xmit => '1', ERROR => '0'),
9	 => ( DATA => X"7a", VALID => '1', xmit => '1', ERROR => '0'),
10	 => ( DATA => X"e9", VALID => '1', xmit => '1', ERROR => '0'),
11	 => ( DATA => X"63", VALID => '1', xmit => '1', ERROR => '0'),
12	 => ( DATA => X"08", VALID => '1', xmit => '1', ERROR => '0'),
13	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
14	 => ( DATA => X"45", VALID => '1', xmit => '1', ERROR => '0'),
15	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
16	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
17	 => ( DATA => X"54", VALID => '1', xmit => '1', ERROR => '0'),
18	 => ( DATA => X"cf", VALID => '1', xmit => '1', ERROR => '0'),
19	 => ( DATA => X"4d", VALID => '1', xmit => '1', ERROR => '0'),
20	 => ( DATA => X"40", VALID => '1', xmit => '1', ERROR => '0'),
21	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
22	 => ( DATA => X"40", VALID => '1', xmit => '1', ERROR => '0'),
23	 => ( DATA => X"01", VALID => '1', xmit => '1', ERROR => '0'),
24	 => ( DATA => X"21", VALID => '1', xmit => '1', ERROR => '0'),
25	 => ( DATA => X"f1", VALID => '1', xmit => '1', ERROR => '0'),
26	 => ( DATA => X"c0", VALID => '1', xmit => '1', ERROR => '0'),
27	 => ( DATA => X"a8", VALID => '1', xmit => '1', ERROR => '0'),
28	 => ( DATA => X"64", VALID => '1', xmit => '1', ERROR => '0'),
29	 => ( DATA => X"09", VALID => '1', xmit => '1', ERROR => '0'),
30	 => ( DATA => X"c0", VALID => '1', xmit => '1', ERROR => '0'),
31	 => ( DATA => X"a8", VALID => '1', xmit => '1', ERROR => '0'),
32	 => ( DATA => X"64", VALID => '1', xmit => '1', ERROR => '0'),
33	 => ( DATA => X"10", VALID => '1', xmit => '1', ERROR => '0'),
34	 => ( DATA => X"08", VALID => '1', xmit => '1', ERROR => '0'),
35	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
36	 => ( DATA => X"dd", VALID => '1', xmit => '1', ERROR => '0'),
37	 => ( DATA => X"96", VALID => '1', xmit => '1', ERROR => '0'),
38	 => ( DATA => X"0b", VALID => '1', xmit => '1', ERROR => '0'),
39	 => ( DATA => X"49", VALID => '1', xmit => '1', ERROR => '0'),
40	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
41	 => ( DATA => X"01", VALID => '1', xmit => '1', ERROR => '0'),
42	 => ( DATA => X"8f", VALID => '1', xmit => '1', ERROR => '0'),
43	 => ( DATA => X"b1", VALID => '1', xmit => '1', ERROR => '0'),
44	 => ( DATA => X"b2", VALID => '1', xmit => '1', ERROR => '0'),
45	 => ( DATA => X"52", VALID => '1', xmit => '1', ERROR => '0'),
46	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
47	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
48	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
49	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
50	 => ( DATA => X"05", VALID => '1', xmit => '1', ERROR => '0'),
51	 => ( DATA => X"48", VALID => '1', xmit => '1', ERROR => '0'),
52	 => ( DATA => X"09", VALID => '1', xmit => '1', ERROR => '0'),
53	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
54	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
55	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
56	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
57	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
58	 => ( DATA => X"10", VALID => '1', xmit => '1', ERROR => '0'),
59	 => ( DATA => X"11", VALID => '1', xmit => '1', ERROR => '0'),
60	 => ( DATA => X"12", VALID => '1', xmit => '1', ERROR => '0'),
61	 => ( DATA => X"13", VALID => '1', xmit => '1', ERROR => '0'),
62	 => ( DATA => X"14", VALID => '1', xmit => '1', ERROR => '0'),
63	 => ( DATA => X"15", VALID => '1', xmit => '1', ERROR => '0'),
64	 => ( DATA => X"16", VALID => '1', xmit => '1', ERROR => '0'),
65	 => ( DATA => X"17", VALID => '1', xmit => '1', ERROR => '0'),
66	 => ( DATA => X"18", VALID => '1', xmit => '1', ERROR => '0'),
67	 => ( DATA => X"19", VALID => '1', xmit => '1', ERROR => '0'),
68	 => ( DATA => X"1a", VALID => '1', xmit => '1', ERROR => '0'),
69	 => ( DATA => X"1b", VALID => '1', xmit => '1', ERROR => '0'),
70	 => ( DATA => X"1c", VALID => '1', xmit => '1', ERROR => '0'),
71	 => ( DATA => X"1d", VALID => '1', xmit => '1', ERROR => '0'),
72	 => ( DATA => X"1e", VALID => '1', xmit => '1', ERROR => '0'),
73	 => ( DATA => X"1f", VALID => '1', xmit => '1', ERROR => '0'),
74	 => ( DATA => X"20", VALID => '1', xmit => '1', ERROR => '0'),
75	 => ( DATA => X"21", VALID => '1', xmit => '1', ERROR => '0'),
76	 => ( DATA => X"22", VALID => '1', xmit => '1', ERROR => '0'),
77	 => ( DATA => X"23", VALID => '1', xmit => '1', ERROR => '0'),
78	 => ( DATA => X"24", VALID => '1', xmit => '1', ERROR => '0'),
79	 => ( DATA => X"25", VALID => '1', xmit => '1', ERROR => '0'),
80	 => ( DATA => X"26", VALID => '1', xmit => '1', ERROR => '0'),
81	 => ( DATA => X"27", VALID => '1', xmit => '1', ERROR => '0'),
82	 => ( DATA => X"28", VALID => '1', xmit => '1', ERROR => '0'),
83	 => ( DATA => X"29", VALID => '1', xmit => '1', ERROR => '0'),
84	 => ( DATA => X"2a", VALID => '1', xmit => '1', ERROR => '0'),
85	 => ( DATA => X"2b", VALID => '1', xmit => '1', ERROR => '0'),
86	 => ( DATA => X"2c", VALID => '1', xmit => '1', ERROR => '0'),
87	 => ( DATA => X"2d", VALID => '1', xmit => '1', ERROR => '0'),
88	 => ( DATA => X"2e", VALID => '1', xmit => '1', ERROR => '0'),
89	 => ( DATA => X"2f", VALID => '1', xmit => '1', ERROR => '0'),
90	 => ( DATA => X"30", VALID => '1', xmit => '1', ERROR => '0'),
91	 => ( DATA => X"31", VALID => '1', xmit => '1', ERROR => '0'),
92	 => ( DATA => X"32", VALID => '1', xmit => '1', ERROR => '0'),
93	 => ( DATA => X"33", VALID => '1', xmit => '1', ERROR => '0'),
94	 => ( DATA => X"34", VALID => '1', xmit => '1', ERROR => '0'),
95	 => ( DATA => X"35", VALID => '1', xmit => '1', ERROR => '0'),
96	 => ( DATA => X"36", VALID => '1', xmit => '1', ERROR => '0'),
97	 => ( DATA => X"37", VALID => '1', xmit => '1', ERROR => '0'),
   others  => ( DATA => X"00", VALID => '0', xmit => '0', ERROR => '0')),
  -- No error in this frame
  bad_frame => false),


-------------
-- Frame 2
-------------
2          => (
  columns  => (
0	 => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
1	 => ( DATA => X"0d", VALID => '1', xmit => '1', ERROR => '0'),
2	 => ( DATA => X"db", VALID => '1', xmit => '1', ERROR => '0'),
3	 => ( DATA => X"a1", VALID => '1', xmit => '1', ERROR => '0'),
4	 => ( DATA => X"15", VALID => '1', xmit => '1', ERROR => '0'),
5	 => ( DATA => X"30", VALID => '1', xmit => '1', ERROR => '0'),
6	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
7	 => ( DATA => X"30", VALID => '1', xmit => '1', ERROR => '0'),
8	 => ( DATA => X"48", VALID => '1', xmit => '1', ERROR => '0'),
9	 => ( DATA => X"7a", VALID => '1', xmit => '1', ERROR => '0'),
10	 => ( DATA => X"e9", VALID => '1', xmit => '1', ERROR => '0'),
11	 => ( DATA => X"63", VALID => '1', xmit => '1', ERROR => '0'),
12	 => ( DATA => X"08", VALID => '1', xmit => '1', ERROR => '0'),
13	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
14	 => ( DATA => X"45", VALID => '1', xmit => '1', ERROR => '0'),
15	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
16	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
17	 => ( DATA => X"60", VALID => '1', xmit => '1', ERROR => '0'),
18	 => ( DATA => X"82", VALID => '1', xmit => '1', ERROR => '0'),
19	 => ( DATA => X"3b", VALID => '1', xmit => '1', ERROR => '0'),
20	 => ( DATA => X"40", VALID => '1', xmit => '1', ERROR => '0'),
21	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
22	 => ( DATA => X"40", VALID => '1', xmit => '1', ERROR => '0'),
23	 => ( DATA => X"11", VALID => '1', xmit => '1', ERROR => '0'),
24	 => ( DATA => X"6e", VALID => '1', xmit => '1', ERROR => '0'),
25	 => ( DATA => X"e7", VALID => '1', xmit => '1', ERROR => '0'),
26	 => ( DATA => X"c0", VALID => '1', xmit => '1', ERROR => '0'),
27	 => ( DATA => X"a8", VALID => '1', xmit => '1', ERROR => '0'),
28	 => ( DATA => X"64", VALID => '1', xmit => '1', ERROR => '0'),
29	 => ( DATA => X"09", VALID => '1', xmit => '1', ERROR => '0'),
30	 => ( DATA => X"c0", VALID => '1', xmit => '1', ERROR => '0'),
31	 => ( DATA => X"a8", VALID => '1', xmit => '1', ERROR => '0'),
32	 => ( DATA => X"64", VALID => '1', xmit => '1', ERROR => '0'),
33	 => ( DATA => X"10", VALID => '1', xmit => '1', ERROR => '0'),
34	 => ( DATA => X"9c", VALID => '1', xmit => '1', ERROR => '0'),
35	 => ( DATA => X"af", VALID => '1', xmit => '1', ERROR => '0'),
36	 => ( DATA => X"c3", VALID => '1', xmit => '1', ERROR => '0'),
37	 => ( DATA => X"51", VALID => '1', xmit => '1', ERROR => '0'),
38	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
39	 => ( DATA => X"4c", VALID => '1', xmit => '1', ERROR => '0'),
40	 => ( DATA => X"49", VALID => '1', xmit => '1', ERROR => '0'),
41	 => ( DATA => X"c8", VALID => '1', xmit => '1', ERROR => '0'),
42	 => ( DATA => X"f0", VALID => '1', xmit => '1', ERROR => '0'),
43	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
44	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
45	 => ( DATA => X"20", VALID => '1', xmit => '1', ERROR => '0'),
46	 => ( DATA => X"f1", VALID => '1', xmit => '1', ERROR => '0'),
47	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
48	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
49	 => ( DATA => X"20", VALID => '1', xmit => '1', ERROR => '0'),
50	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
51	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
52	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
53	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
54	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
55	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
56	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
57	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
58	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
59	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
60	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
61	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
62	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
63	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
64	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
65	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
66	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
67	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
68	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
69	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
70	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
71	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
72	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
73	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
74	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
75	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
76	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
77	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
78	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
79	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
80	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
81	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
82	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
83	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
84	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
85	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
86	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
87	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
88	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
89	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
90	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
91	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
92	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
93	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
94	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
95	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
96	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
97	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
98	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
99	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
100	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
101	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
102	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
103	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
104	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
105	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
106	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
107	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
108	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
109	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   others  => ( DATA => X"00", VALID => '0', xmit => '0', ERROR => '0')),

   -- Error this frame
--   bad_frame => true),
  bad_frame => false),


-------------
-- Frame 3
-------------
3          => (
  columns  => (
0	 => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
1	 => ( DATA => X"0d", VALID => '1', xmit => '1', ERROR => '0'),
2	 => ( DATA => X"db", VALID => '1', xmit => '1', ERROR => '0'),
3	 => ( DATA => X"a1", VALID => '1', xmit => '1', ERROR => '0'),
4	 => ( DATA => X"15", VALID => '1', xmit => '1', ERROR => '0'),
5	 => ( DATA => X"30", VALID => '1', xmit => '1', ERROR => '0'),
6	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
7	 => ( DATA => X"30", VALID => '1', xmit => '1', ERROR => '0'),
8	 => ( DATA => X"48", VALID => '1', xmit => '1', ERROR => '0'),
9	 => ( DATA => X"7a", VALID => '1', xmit => '1', ERROR => '0'),
10	 => ( DATA => X"e9", VALID => '1', xmit => '1', ERROR => '0'),
11	 => ( DATA => X"63", VALID => '1', xmit => '1', ERROR => '0'),
12	 => ( DATA => X"08", VALID => '1', xmit => '1', ERROR => '0'),
13	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
14	 => ( DATA => X"45", VALID => '1', xmit => '1', ERROR => '0'),
15	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
16	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
17	 => ( DATA => X"28", VALID => '1', xmit => '1', ERROR => '0'),
18	 => ( DATA => X"c1", VALID => '1', xmit => '1', ERROR => '0'),
19	 => ( DATA => X"05", VALID => '1', xmit => '1', ERROR => '0'),
20	 => ( DATA => X"40", VALID => '1', xmit => '1', ERROR => '0'),
21	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
22	 => ( DATA => X"40", VALID => '1', xmit => '1', ERROR => '0'),
23	 => ( DATA => X"11", VALID => '1', xmit => '1', ERROR => '0'),
24	 => ( DATA => X"30", VALID => '1', xmit => '1', ERROR => '0'),
25	 => ( DATA => X"55", VALID => '1', xmit => '1', ERROR => '0'),
26	 => ( DATA => X"c0", VALID => '1', xmit => '1', ERROR => '0'),
27	 => ( DATA => X"a8", VALID => '1', xmit => '1', ERROR => '0'),
28	 => ( DATA => X"64", VALID => '1', xmit => '1', ERROR => '0'),
29	 => ( DATA => X"09", VALID => '1', xmit => '1', ERROR => '0'),
30	 => ( DATA => X"c0", VALID => '1', xmit => '1', ERROR => '0'),
31	 => ( DATA => X"a8", VALID => '1', xmit => '1', ERROR => '0'),
32	 => ( DATA => X"64", VALID => '1', xmit => '1', ERROR => '0'),
33	 => ( DATA => X"10", VALID => '1', xmit => '1', ERROR => '0'),
34	 => ( DATA => X"a1", VALID => '1', xmit => '1', ERROR => '0'),
35	 => ( DATA => X"a5", VALID => '1', xmit => '1', ERROR => '0'),
36	 => ( DATA => X"c3", VALID => '1', xmit => '1', ERROR => '0'),
37	 => ( DATA => X"51", VALID => '1', xmit => '1', ERROR => '0'),
38	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
39	 => ( DATA => X"14", VALID => '1', xmit => '1', ERROR => '0'),
40	 => ( DATA => X"49", VALID => '1', xmit => '1', ERROR => '0'),
41	 => ( DATA => X"90", VALID => '1', xmit => '1', ERROR => '0'),
42	 => ( DATA => X"f0", VALID => '1', xmit => '1', ERROR => '0'),
43	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
44	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
45	 => ( DATA => X"20", VALID => '1', xmit => '1', ERROR => '0'),
46	 => ( DATA => X"0f", VALID => '1', xmit => '1', ERROR => '0'),
47	 => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
48	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
49	 => ( DATA => X"20", VALID => '1', xmit => '1', ERROR => '0'),
50	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
51	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
52	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
53	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
54	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
55	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
56	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
57	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
58	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
59	 => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   others  => ( DATA => X"00", VALID => '0', xmit => '0', ERROR => '0')),

  -- No error in this frame
  bad_frame => false),

-------------
-- Frame 4
-------------
4          => (
  columns  => (
	0      => ( DATA => X"DA", VALID => '1', xmit => '1', ERROR => '0'), -- Destination Address (DA)
	1      => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
	2      => ( DATA => X"03", VALID => '1', xmit => '1', ERROR => '0'),
	3      => ( DATA => X"04", VALID => '1', xmit => '1', ERROR => '0'),
	4      => ( DATA => X"05", VALID => '1', xmit => '1', ERROR => '0'),
	5      => ( DATA => X"06", VALID => '1', xmit => '1', ERROR => '0'),
	6      => ( DATA => X"5A", VALID => '1', xmit => '1', ERROR => '0'), -- Source Address (5A)
	7      => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
	8      => ( DATA => X"03", VALID => '1', xmit => '1', ERROR => '0'),
	9      => ( DATA => X"04", VALID => '1', xmit => '1', ERROR => '0'),
   10      => ( DATA => X"05", VALID => '1', xmit => '1', ERROR => '0'),
   11      => ( DATA => X"06", VALID => '1', xmit => '1', ERROR => '0'),
   12      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   13      => ( DATA => X"2D", VALID => '1', xmit => '1', ERROR => '0'), -- Length/Type = Length = 45
   14      => ( DATA => X"01", VALID => '1', xmit => '1', ERROR => '0'), -- Therefore padding is required
   15      => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
   16      => ( DATA => X"03", VALID => '1', xmit => '1', ERROR => '0'),
   17      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'), 
   18      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   19      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   20      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   21      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   22      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   23      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   24      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   25      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   26      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   27      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   28      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   29      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   30      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   31      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   32      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   33      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   34      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   35      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   36      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   37      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   38      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   39      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   40      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   41      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   42      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   43      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   44      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   45      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   46      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   47      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   48      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   49      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   50      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   51      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   52      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   53      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   54      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   55      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   56      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   57      => ( DATA => X"B0", VALID => '1', xmit => '1', ERROR => '0'),
   58      => ( DATA => X"AA", VALID => '1', xmit => '1', ERROR => '0'),
   59      => ( DATA => X"AA", VALID => '1', xmit => '0', ERROR => '0'),
   others  => ( DATA => X"00", VALID => '0', xmit => '0', ERROR => '0')),

  -- No error in this frame
  bad_frame => false),


-- -------------
-- -- Frame 5
-- -------------
5         => (
  columns  => (
	0      => ( DATA => X"DA", VALID => '1', xmit => '1', ERROR => '0'), -- Destination Address (DA)
	1      => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
	2      => ( DATA => X"03", VALID => '1', xmit => '1', ERROR => '0'),
	3      => ( DATA => X"04", VALID => '1', xmit => '1', ERROR => '0'),
	4      => ( DATA => X"05", VALID => '1', xmit => '1', ERROR => '0'),
	5      => ( DATA => X"06", VALID => '1', xmit => '1', ERROR => '0'),
	6      => ( DATA => X"5A", VALID => '1', xmit => '1', ERROR => '0'), -- Source Address (5A)
	7      => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
	8      => ( DATA => X"03", VALID => '1', xmit => '1', ERROR => '0'),
	9      => ( DATA => X"04", VALID => '1', xmit => '1', ERROR => '0'),
   10      => ( DATA => X"05", VALID => '1', xmit => '1', ERROR => '0'),
   11      => ( DATA => X"06", VALID => '1', xmit => '1', ERROR => '0'),
   12      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   13      => ( DATA => X"2C", VALID => '1', xmit => '1', ERROR => '0'), -- Length/Type = Length = 44
   14      => ( DATA => X"01", VALID => '1', xmit => '1', ERROR => '0'), -- Therefore padding is required
   15      => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
   16      => ( DATA => X"03", VALID => '1', xmit => '1', ERROR => '0'),
   17      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'), 
   18      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   19      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   20      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   21      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   22      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   23      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   24      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   25      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   26      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   27      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   28      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   29      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   30      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   31      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   32      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   33      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   34      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   35      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   36      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   37      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   38      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   39      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   40      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   41      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   42      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   43      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   44      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   45      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   46      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   47      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   48      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   49      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   50      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   51      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   52      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   53      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   54      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   55      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   56      => ( DATA => X"CB", VALID => '1', xmit => '1', ERROR => '0'),
   57      => ( DATA => X"AB", VALID => '1', xmit => '1', ERROR => '0'),
   58      => ( DATA => X"AB", VALID => '1', xmit => '0', ERROR => '0'),
   59      => ( DATA => X"AB", VALID => '1', xmit => '0', ERROR => '0'),
   others  => ( DATA => X"00", VALID => '0', xmit => '0', ERROR => '0')),

  -- No error in this frame
  bad_frame => false));


------------------------------------------------------------------------------
-- CRC engine
------------------------------------------------------------------------------
function calc_crc (data : in std_logic_vector;
				 fcs  : in std_logic_vector)
return std_logic_vector is

variable crc          : std_logic_vector(31 downto 0);
variable crc_feedback : std_logic;
begin

crc := not fcs;

for I in 0 to 7 loop
  crc_feedback      := crc(0) xor data(I);

  crc(4 downto 0)   := crc(5 downto 1);
  crc(5)            := crc(6)  xor crc_feedback;
  crc(7 downto 6)   := crc(8 downto 7);
  crc(8)            := crc(9)  xor crc_feedback;
  crc(9)            := crc(10) xor crc_feedback;
  crc(14 downto 10) := crc(15 downto 11);
  crc(15)           := crc(16) xor crc_feedback;
  crc(18 downto 16) := crc(19 downto 17);
  crc(19)           := crc(20) xor crc_feedback;
  crc(20)           := crc(21) xor crc_feedback;
  crc(21)           := crc(22) xor crc_feedback;
  crc(22)           := crc(23);
  crc(23)           := crc(24) xor crc_feedback;
  crc(24)           := crc(25) xor crc_feedback;
  crc(25)           := crc(26);
  crc(26)           := crc(27) xor crc_feedback;
  crc(27)           := crc(28) xor crc_feedback;
  crc(28)           := crc(29);
  crc(29)           := crc(30) xor crc_feedback;
  crc(30)           := crc(31) xor crc_feedback;
  crc(31)           :=             crc_feedback;
end loop;
-- return the CRC result
return not crc;
end calc_crc;


signal rstn : std_logic;
signal rst : std_logic;
signal clk125 : std_logic;
signal gtx_clk : std_logic;
signal sysclk_p : std_logic;
signal sysclk_n : std_logic;
  
signal gmii_tx_en           : std_logic:='0';
signal gmii_tx_er           : std_logic:='0';
signal gmii_txd             : std_logic_vector(7 downto 0) := (others => '0');
alias gmii_rx_clk          : std_logic is clk125;
signal gmii_rx_dv           : std_logic := '0';
signal gmii_rx_er           : std_logic := '0';
signal gmii_rxd             : std_logic_vector(7 downto 0) := (others => '0');
		
signal gtp_clkp, gtp_clkn: std_logic;
signal gtp_txp, gtp_txn: std_logic;
signal gtp_rxp, gtp_rxn: std_logic;

constant dly : time := 0 ns; 
constant gtx_period : time := 5 ns;
constant gtp_period : time := 8 ns;

begin
lbl_DUT :  top_ipbus_lpsc_basex generic map(
	IN_SIMULATION=>true,
	SIM_GTPRESET_SPEEDUP=>1,
	SIMUL_DST_MAC_NUMBER=>x"AAAA_BBBB_CCCC",
	CLOCK_FREQ=>125000000,							-- freq of data_in_clk -- needed to timout cntr
	ARP_TIMEOUT=>60,								-- ARP response timeout (s)
	ARP_MAX_PKT_TMO=>5,							-- # wrong nwk pkts received before set error
	MAX_ARP_ENTRIES=>255)									-- max entries in the ARP store
port map(
	sysclk_p=>sysclk_p,
   	sysclk_n=>sysclk_n,
	leds=>open,
	gtp_clkp=>gtp_clkp,
	gtp_clkn=>gtp_clkn,

	gtp_txp=>gtp_txp,
	gtp_txn=>gtp_txn,

-- 	gtp_txp=>open,
-- 	gtp_txn=>open,

	gtp_rxp=>gtp_rxp,
   	gtp_rxn=>gtp_rxn,

	sfp_los=>'0',
	user_sma_gpio_n=>open,
	user_sma_gpio_p=>open,
-- 	dip_switch=>"0000"
	dip_switch=>"0011"
	);
  
p_rst : process
	begin
	rstn<='0';
	wait for 20 ns;

	rstn<='1';
	wait;
end process p_rst;
rst<=not(rstn);

p_gtx_clk : process
	begin
	sysclk_p <= '0';
	sysclk_n <= '1';
	wait for gtx_period/2;

	sysclk_p <= '1';
	sysclk_n <= '0';
	wait for gtx_period/2;
end process p_gtx_clk;

p_gtp_clk : process
	begin
	gtp_clkp <= '0';
	gtp_clkn <= '1';
	wait for gtp_period/2;

	gtp_clkp <= '1';
	gtp_clkn <= '0';
	wait for gtp_period/2;
end process p_gtp_clk;

p_clk125 : process
	begin
	clk125 <= '0';
	wait for 4 ns;

	clk125 <= '1';
	wait for 4 ns;
end process p_clk125;

------------------------------------------------------------------------------
-- PHY simulator (same as RTL)
------------------------------------------------------------------------------
phy: entity work.gig_eth_pcs_pma_v11_3_block generic map (SIM_GTPRESET_SPEEDUP=>1)
	port map(
	gtpclkout => open,
-- 	gtpreset0 => '0',  -- rely upon powerup reset for now
	gtpreset0 => rst,  -- rely upon powerup reset for now
	gmii_txd0 => gmii_txd,
	gmii_tx_en0 => gmii_tx_en,
	gmii_tx_er0 => gmii_tx_er,
	gmii_rxd0 => gmii_rxd,
	gmii_rx_dv0 => gmii_rx_dv,
	gmii_rx_er0 => gmii_rx_er,
	gmii_isolate0 => open,
 	configuration_vector0 => "00000", --disable autoneg 
-- 	configuration_vector0 => "10000", --allow autoneg
	link_timer_value0=>conv_std_logic_vector(4000,9),  -- Programmable Auto-Negotiation Link Timer Control
	an_interrupt0=>open,
	an_adv_config_vector0=>an_adv_config_vector0,
	an_restart_config0=>'1',
	status_vector0 => open,
	test=>open,
	reset0 => rst,
	signal_detect0 => '1',
	clkin => clk125,
	userclk2 => clk125,
	txp0 => gtp_rxp,
	txn0 => gtp_rxn,
	rxp0 => gtp_txp,
	rxn0 => gtp_txn,
	txp1 => open,
	txn1 => open,
	rxp1 => '0',
	rxn1 => '0'
	);

an_adv_config_vector0(0)<='0'; -- reserved
an_adv_config_vector0(4 downto 1)<="0000"; -- reserved
an_adv_config_vector0(5)<='1'; -- full duplex mode
an_adv_config_vector0(6)<='0'; -- reserved
an_adv_config_vector0(8 downto 7)<="00"; -- no pause
an_adv_config_vector0(9)<='0'; -- reserved
an_adv_config_vector0(11 downto 10)<="00"; -- reserved
an_adv_config_vector0(13 downto 12)<="00"; -- no error (remote fault)
an_adv_config_vector0(14)<='0'; -- reserved
an_adv_config_vector0(15)<='0'; -- reserved


------------------------------------------------------------------------------
-- Stimulus process. This process will inject frames of data into the
-- PHY side of the receiver.
------------------------------------------------------------------------------
p_stimulus : process
	----------------------------------------------------------
	-- Procedure to inject a frame into the receiver at 1Gb/s
	----------------------------------------------------------
	procedure send_frame_1g (current_frame : in natural) is
	variable current_col   : natural := 0;  -- Column counter within frame
	variable fcs           : std_logic_vector(31 downto 0);
	begin

	wait until clk125'event and clk125 = '1';

	-- Reset the FCS calculation
	fcs         := (others => '0');

	-- Adding the preamble field
	for j in 0 to 6 loop
	gmii_txd   <= "01010101" after dly;
	gmii_tx_en <= '1' after dly;
	gmii_tx_er <= '0' after dly;
	wait until clk125'event and clk125 = '1';
	end loop;

	-- Adding the Start of Frame Delimiter (SFD)
	gmii_txd   <= "11010101" after dly;
	gmii_tx_en <= '1' after dly;
	wait until clk125'event and clk125 = '1';

	current_col := 0;
	gmii_txd     <= to_stdlogicvector(frame_data(current_frame).columns(current_col).data) after dly;
	gmii_tx_en   <= to_stdUlogic(frame_data(current_frame).columns(current_col).valid) after dly;
	gmii_tx_er   <= to_stdUlogic(frame_data(current_frame).columns(current_col).error) after dly;
	fcs          := calc_crc(to_stdlogicvector(frame_data(current_frame).columns(current_col).data), fcs);

	wait until clk125'event and clk125 = '1';

	current_col := current_col + 1;
	-- loop over columns in frame.
	while frame_data(current_frame).columns(current_col).valid /= '0' loop
	-- send one column of data
	gmii_txd   <= to_stdlogicvector(frame_data(current_frame).columns(current_col).data) after dly;
	gmii_tx_en <= to_stdUlogic(frame_data(current_frame).columns(current_col).valid) after dly;
	gmii_tx_er   <= to_stdUlogic(frame_data(current_frame).columns(current_col).error) after dly;
	fcs          := calc_crc(to_stdlogicvector(frame_data(current_frame).columns(current_col).data), fcs);

	current_col := current_col + 1;
	wait until clk125'event and clk125 = '1';

	end loop;

	-- Send the CRC.
	for j in 0 to 3 loop
	gmii_txd   <= fcs(((8*j)+7) downto (8*j)) after dly;
	gmii_tx_en <= '1' after dly;
	gmii_tx_er <= '0' after dly;
	wait until clk125'event and clk125 = '1';
	end loop;

	-- Clear the data lines.
	gmii_txd   <= (others => '0') after dly;
	gmii_tx_en <=  '0' after dly;

	-- Adding the minimum Interframe gap for a receiver (8 idles)
	for j in 0 to 7 loop
	wait until clk125'event and clk125 = '1';
	end loop;

	end send_frame_1g;
begin
	-- Send four frames through the MAC and Design Exampled
	-- at each state Ethernet speed
	--      -- frame 0 = minimum length frame
	--      -- frame 1 = type frame
	--      -- frame 2 = errored frame
	--      -- frame 3 = padded frame
	-------------------------------------------------------


	-- 1 Gb/s speed
	-------------------------------------------------------
	-- Wait for the internal resets to settle
	wait for 800 ns;

	assert false
	report "Sending four frames at 1Gb/s..." & cr
	severity note;

-- 	wait for 401 us;
-- 	report "Simulation stopped"
-- 	severity failure;

-- 	for current_frame in frame_data'low to frame_data'high loop
	for current_frame in frame_data'low to 3 loop
	send_frame_1g(current_frame);
	wait for 1100 ns;
	end loop;

	wait for 3 us;
	-- Wait for 1G monitor process to complete.
	-- Our work here is done
	assert false
	report "Simulation stopped"
	severity failure;
end process p_stimulus;

end simu;

