library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity vect_domain_xfer is
generic(size : positive);
port(
	clkin : in std_logic;
	rstin : in std_logic;
	vect_in : in std_logic_vector(size-1 downto 0);

	clkout : in std_logic;
	rstout : in std_logic;
	vect_out : out std_logic_vector(size-1 downto 0)
	);
end vect_domain_xfer;

architecture RTL of vect_domain_xfer is
function conv_seq_to_gray(input : std_logic_vector) return std_logic_vector is
variable int_vect : std_logic_vector(input'length-1 downto 0);
begin
	for i in 0 to input'length-2 loop
	int_vect(i):=input(i+1) XOR input(i);
	end loop;
	int_vect(input'length-1):=input(input'length-1);

	return int_vect;		
end conv_seq_to_gray;

function conv_gray_to_seq(input : std_logic_vector) return std_logic_vector is
variable int_vect : std_logic_vector(input'length-1 downto 0);
variable int_xor : std_logic;
begin
	int_xor:=input(input'length-1);
	int_vect(input'length-1):=int_xor;

	for i in input'length-2 downto 0 loop
	int_xor:=int_xor xor input(i);
	int_vect(i):=int_xor;
	end loop;

	return int_vect;
end conv_gray_to_seq;

signal vect_in_gray : std_logic_vector(size-1 downto 0);
signal vect_in_gray_1ck : std_logic_vector(size-1 downto 0);
signal vect_in_gray_2ck : std_logic_vector(size-1 downto 0);

constant MAX_min_one   : std_logic_vector(size-1 downto 0):=(others=>'1');
constant MAX_min_two   : std_logic_vector(size-1 downto 0):=conv_std_logic_vector(2**size-2,size);
constant MAX_min_three : std_logic_vector(size-1 downto 0):=conv_std_logic_vector(2**size-3,size);

begin
process(clkin,rstin)
begin
	if rstin='1' then
		vect_in_gray<=(others=>'0');
	elsif rising_edge(clkin) then
		vect_in_gray<=conv_seq_to_gray(vect_in);
	end if;
end process;


process(clkout,rstout)
begin
	if rstout='1' then
		vect_in_gray_1ck<=(others=>'0');
		vect_in_gray_2ck<=(others=>'0');
		vect_out<=(others=>'0');

	elsif rising_edge(clkout) then
		vect_in_gray_1ck<=vect_in_gray;
		vect_in_gray_2ck<=vect_in_gray_1ck;

		vect_out<=conv_gray_to_seq(vect_in_gray_2ck);
	end if;
end process;
end RTL;


