----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    08:03:30 06/04/2011 
-- Design Name: 
-- Module Name:    tx_arbitrator - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 	arbitrate between two sources that want to transmit onto a bus
--						handles arbitration and multiplexing
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Revision 0.02 - Made sticky on port M1 to optimise access on this port and allow immediate grant
-- Revision 0.03 - Added first
-- Additional Comments: 
--
-- modified OB :
-- Interface with vector
-- first priority on port 0
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use work.pack_ipv4_types.all;

entity mac_tx_arbitrator is
generic(nb_src : positive :=2);
port (
	clk				: in std_logic;
	reset				: in std_logic;

	src : in arbitrator_in_vect_type(nb_src-1 downto 0); -- all input to aritrate: priority on 0
	grant : out std_logic_vector(nb_src-1 downto 0); -- all input to aritrate: priority on 0

	data         	: out  std_logic_vector(7 downto 0);	-- data byte to tx
	valid        	: out  std_logic;							-- tdata is valid
	first         	: out  std_logic;							-- indicates first byte of frame
	last         	: out  std_logic							-- indicates last byte of frame
	);
end mac_tx_arbitrator;

architecture Behavioral of mac_tx_arbitrator is
type def_state_arb is (idle,busy);
signal state_arb : def_state_arb;

signal sel : natural range 0 to nb_src-1;

begin

process(clk)
begin
	if rising_edge(clk) then
		if reset = '1' then
			sel<=0;
			for i in 0 to nb_src-1 loop
				grant(i)<='0';
			end loop;
			state_arb<=idle;
		else
			case state_arb is
				when idle =>
					if src(sel).req='1' then
						grant(sel)<='1';
						state_arb<=busy;
					elsif sel=nb_src-1 then
						sel<=0;
					else
						sel<=sel+1;
					end if;

				when busy =>
					if src(sel).last='1' then
						grant(sel)<='0';
						sel<=0;
						state_arb<=idle;
					end if;
			end case;

		end if;
	end if;
end process;

-- multiplexer
data <= src(sel).data;
valid <= src(sel).valid;
first <= src(sel).first;
last <= src(sel).last;
end Behavioral;
