library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;
use ieee.std_logic_unsigned.all;

entity tb_arp_store is
end tb_arp_store;

architecture simu of tb_arp_store is
component arp_store
generic (
    MAX_ARP_ENTRIES : integer := 16 -- max entries in the store
    );
port (
    -- system signals
    clk         : in  std_logic;
    reset       : in  std_logic;

    -- arp lookup signals
    req_ip    : in  std_logic_vector(31 downto 0);
	do_lookup : in std_logic;
	lookup_done : out std_logic;
    rslt_mac    : out  std_logic_vector(47 downto 0);
	req_found : out std_logic;

	-- arp table write signals
    wr_ip    : in  std_logic_vector(31 downto 0);
    wr_mac    : in  std_logic_vector(47 downto 0);
	insert_arp : in std_logic;
	insert_done : out std_logic;

    -- control and status signals
    clear_store : in  std_logic;              -- erase all entries
    entry_count : out std_logic_vector(7 downto 0)   -- how many entries currently in store
    );
end component;

signal clk         : std_logic;
signal reset       : std_logic;
signal req_ip    : std_logic_vector(31 downto 0);
signal do_lookup :std_logic;
signal lookup_done :std_logic;
signal rslt_mac    : std_logic_vector(47 downto 0);
signal req_found :std_logic;
signal wr_ip    : std_logic_vector(31 downto 0);
signal wr_mac    : std_logic_vector(47 downto 0);
signal insert_arp :std_logic;
signal insert_done :std_logic;
signal clear_store : std_logic;
signal entry_count :std_logic_vector(7 downto 0);

begin
lbl_DUT : arp_store generic map(MAX_ARP_ENTRIES=>4)
port map(
    -- system signals
    clk=>clk,
    reset=>reset,

    -- arp lookup signals
    req_ip=>req_ip,
	do_lookup=>do_lookup,
	lookup_done=>lookup_done,
    rslt_mac=>rslt_mac,
	req_found=>req_found,

	-- arp table write signals
    wr_ip=>wr_ip,
    wr_mac=>wr_mac,
	insert_arp=>insert_arp,
	insert_done=>insert_done,

    -- control and status signals
    clear_store=>clear_store,
    entry_count=>entry_count
    );
process
begin
	clk<='0';
	wait for 4 ns;

	clk<='1';
	wait for 4 ns;
end process;
-------------------------------------------------------------------------------
-- main
-------------------------------------------------------------------------------
process
variable var_ip : unsigned(31 downto 0);
variable var_mac : unsigned(47 downto 0);
begin
	reset<='1';
	do_lookup<='0';
	insert_arp<='0';
	clear_store<='0';
	req_ip<=(others=>'0');
	wait for 100 ns;
	
	reset<='0';
	wait for 100 ns;

	wait until rising_edge(clk);
	var_ip:=X"A0A0_A0A3";
	req_ip<=X"A0A0_A0A1";
	do_lookup<='1';

	wait until rising_edge(clk);
	do_lookup<='0';
	wait until rising_edge(clk);

	wait for 100 ns;

	var_mac:=X"B0BA_0000_BEBE";

	-- insert 6 ip
	for i in 0 to 5 loop
		wait until rising_edge(clk);
		var_ip:=x"A0A0_A0A0"+i;
		var_mac:=var_mac+i;

		wr_ip<=std_logic_vector(var_ip);
		wr_mac<=std_logic_vector(var_mac);
		insert_arp<='1';
		wait until rising_edge(clk);
		insert_arp<='0';
		
		wait until rising_edge(clk) and insert_done='1';
		wait for 30 ns;
	end loop;

	wait for 1 us;

	-- resinsert existing IP
	var_mac:=X"B0BA_0000_BEBE";
	wait until rising_edge(clk);
	var_ip:=x"A0A0_A0A0"+4;
	var_mac:=var_mac+4;

	wr_ip<=std_logic_vector(var_ip);
	wr_mac<=std_logic_vector(var_mac);
	insert_arp<='1';
	wait until rising_edge(clk);
	insert_arp<='0';
	
	wait until rising_edge(clk) and insert_done='1';
	wait for 30 ns;

	-- request one existing ip
	wait until rising_edge(clk);
	req_ip<=X"A0A0_A0A3";
	do_lookup<='1';

	wait until rising_edge(clk);
	do_lookup<='0';
	wait until rising_edge(clk) and lookup_done='1';
	wait for 1 us;

	-- request one non existing ip
	wait until rising_edge(clk);
	req_ip<=X"A0AB_A0A3";
	do_lookup<='1';

	wait until rising_edge(clk);
	do_lookup<='0';
	wait until rising_edge(clk) and lookup_done='1';

	-- request one existing ip (3rd, pos 2)
	wait until rising_edge(clk);
	req_ip<=X"A0A0_A0A2";
	do_lookup<='1';

	wait until rising_edge(clk);
	do_lookup<='0';
	wait until rising_edge(clk) and lookup_done='1';
	wait for 1 us;

	-- request one existing ip (2nd, pos 1)
	wait until rising_edge(clk);
	req_ip<=X"A0A0_A0A5";
	do_lookup<='1';

	wait until rising_edge(clk);
	do_lookup<='0';
	wait until rising_edge(clk) and lookup_done='1';
	wait for 1 us;

	-- request one existing ip (1st, pos 0)
	wait until rising_edge(clk);
	req_ip<=X"A0A0_A0A4";
	do_lookup<='1';

	wait until rising_edge(clk);
	do_lookup<='0';
	wait until rising_edge(clk) and lookup_done='1';
	wait for 1 us;

	-- request same existing ip (1st, pos 0)
	wait until rising_edge(clk);
	req_ip<=X"A0A0_A0A4";
	do_lookup<='1';

	wait until rising_edge(clk);
	do_lookup<='0';
	wait until rising_edge(clk) and lookup_done='1';
	wait for 1 us;

	wait for 1 us;
	report "fin de simulation" severity error;
end process;
end simu;
